/*
 * admin.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Administer and create SCCS files.
 *
 */ 

#include "mysc.h"
#include "version.h"
#include "sccsfile.h"
#include "sf-chkmr.h"
#include "fileiter.h"
#include "getopt.h"

char const main_sccs_id[] = "@(#) MySC admin.c 1.8 00/05/12 16:33:51";

void
usage() {
	fprintf(stderr, msgptr(17, 
"usage: %s [-nrzIMTYV] [-a users] [-d flags] [-e users] [-f flags]\n"
"\t[-i file] [-m MRs] [-t file] [-y comments] file ...\n"),
	prg_name);
}

int
main(int argc, char **argv) {
	int c;
	release first_release = NULL;		/* -r */
	int new_sccs_file = 0;			/* -n */
	char const *iname = NULL;		/* -i -I */
	char const *file_comment = NULL;	/* -t -T */
	list<string> set_flags, unset_flags;	/* -f, -d */
	list<string> add_users, erase_users;	/* -a, -e */
	string mrs, comments;			/* -m -M, -y -Y */
	const int check = 0;			/* -h */
	int reset_checksum = 0;			/* -z */
	int return_status = 0;
	
	if (argc > 0) {
		set_prg_name(argv[0]);
	} else {
		set_prg_name("admin");
	}

	setlocale(LC_ALL, "");
	init_msgdb();

	class getopt opts(argc, argv, "ni?Ir:t?Tf:d:a:e:m?My?YhzV");
	for(c = opts.next(); c != getopt::END_OF_ARGUMENTS; c = opts.next()) {
		switch (c) {
		default:
			quit(-2, msgptr(10, "Unsupported option: '%c'"), c);

		case 'n':
			new_sccs_file = 1;
			break;

		case 'i':
			iname = opts.getarg();
			if (iname == NULL) {
		case 'I':
				iname = "-";
			}
			new_sccs_file = 1;
			break;

		case 'r':
			first_release = release(opts.getarg());
			if (!first_release.valid()) {
				quit(-2, msgptr(18, "Invalid release: '%s'"),
				     opts.getarg());
			}
			break;

		case 't':
			file_comment = opts.getarg();
			if (file_comment == NULL) {
		case 'T':
				file_comment = "";
			}
			break;
		       
		case 'f':
			set_flags.add(opts.getarg());
			break;

		case 'd':
			unset_flags.add(opts.getarg());
			break;

		case 'a':
			add_users.add(opts.getarg());
			break;

		case 'e':
			erase_users.add(opts.getarg());
			break;

		case 'm':
			mrs = opts.getarg();
			if (mrs == NULL) {
		case 'M':
				mrs = "";
			}
			break;

		case 'y':
			comments = opts.getarg();
			if (comments == NULL) {
		case 'Y':
				comments = "";
			}
			break;

#if 0
		case 'h':
			check = 1;
			break;
#endif

		case 'z':
			reset_checksum = 1;
			break;

		case 'V':
			version();
			break;
		}
	}

	list<string> mr_list, comment_list;
	if (new_sccs_file) {
		comment_list = split_comments(comments);
		mr_list = split_mrs(mrs);
	}

	sccs_file_iterator iter(argc, argv, opts.get_index());
	if (iter.length_zero()) {
		quit(-2, msgptr(16, "No SCCS file specified."));
	}
	if (new_sccs_file && iname != NULL && iter.length_many()) {
		quit(-1, msgptr(19, "The 'i' keyletter can't be used"
				   " with multiple files."));
	}

	for(; iter != sccs_file_iterator(); ++iter) {
		sccs_name &name = *iter;

		if (reset_checksum && !check) {
			if (!name.valid()) {
				error(msgptr(20, "%s: Not an SCCS file."), 
				      (char const *) name);
				return_status = 1;
			} else {
				if (sccs_file::update_checksum(name) == -1) {
					return_status = 1;
				}
			}
			continue;
		}
			
		/* enum */ sccs_file::_mode mode = sccs_file::UPDATE;
#if 0
		if (check) {
			mode = sccs_file::READ;
		} else
#endif
		if (new_sccs_file) {
			mode = sccs_file::CREATE;
		}

		sccs_file file(name, mode);
		if (file.failed()) {
			return_status = 1;
			continue;
		}

#if 0
		if (check) {
			file.check();
			continue;
		}
#endif

		if (file.admin(file_comment, set_flags, unset_flags,
			       add_users, erase_users) == -1) {
			return 1;
		}

		if (new_sccs_file) {
			if (file.mr_required()) {
				if (stdin_is_a_tty() && mrs == NULL) {
					mrs = prompt_user(msgptr(21, "MRs? "));
					mr_list = split_mrs(mrs);
				}
				if (mr_list.length() == 0) {
					error(msgptr(22, "%s: MR number(s) must"
						     " be supplied."),
					      (char const *) name);
					return_status = 1;
					continue;
				} else if (file.check_mrs(mr_list)) {
					error(msgptr(23, "%s: Invalid MR"
						     " number(s)."),
					      (char const *) name);
					return_status = 1;
					continue;
				}
			}
			if (file.create(first_release, iname, 
					mr_list, comment_list) == -1) {
				return_status = 1;
				continue;
			}
		} else {
			if (file.update() == -1) {
				return_status = 1;
				continue;
			}
		}		
	}

	return return_status;
}

/* Local variables: */
/* mode: c++ */
/* End: */
