/*
 * delta.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Adds new deltas to SCCS files.
 *
 */

#include "mysc.h"
#include "version.h"
#include "getopt.h"
#include "fileiter.h"
#include "pfile.h"
#include "sccsfile.h"
#include "sf-chkmr.h"

char const main_sccs_id[] = "@(#) MySC delta.c 1.11 98/06/06 16:23:04";

void
usage() {
	fprintf(stderr, msgptr(27, 
"usage: %s [-nsMYV] [-m MRs] [-r SID] [-y comments] file ...\n"),
		prg_name);
}

int
main(int argc, char **argv) {
	int c;
	sid rid = NULL;		/* -r */
	int silent = 0;		/* -s */
	int keep_gfile = 0;	/* -n */
#if 0
	sid_list ignore;	/* -g */
#endif
	string mrs;		/* -m -M */
	string comments;	/* -y -Y */
	int null_delta = 0;	/* -N */
	int return_status = 0;

	if (argc > 0) {
		set_prg_name(argv[0]);
	} else {
		set_prg_name("delta");
	}

	setlocale(LC_ALL, "");
	init_msgdb();

	class getopt opts(argc, argv, "r:sng:m?My?YpVN");
	for(c = opts.next(); c != getopt::END_OF_ARGUMENTS; c = opts.next()) {
		switch (c) {
		default:
			quit(-2, msgptr(10, "Unsupported option: '%c'"), c);
		case 'r':
			rid = sid(opts.getarg());
			if (!rid.valid()) {
				quit(-2, msgptr(11, "Invalid SID: '%s'"),
				     opts.getarg());
			}
			break;

		case 's':
			silent = 1;
			break;

		case 'n':
			keep_gfile = 1;
			break;

		case 'N':
			null_delta = 1;
			break;

		case 'm':
			mrs = opts.getarg();
			if (mrs == NULL) {
		case 'M':
				mrs = "";
			}
			break;

		case 'y':
			comments = opts.getarg();
			if (comments == NULL) {
		case 'Y':
				comments = "";
			}
			break;

		case 'V':
			version();
			break;
		}
	}

     	if (silent) {
		stdout_to_null();
	}

	list<string> mr_list, comment_list;
	int first = 1;

	sccs_file_iterator iter(argc, argv, opts.get_index());
	if (iter.length_zero()) {
		quit(-2, msgptr(16, "No SCCS file specified."));
	}

	for(; iter != sccs_file_iterator(); ++iter) {
		sccs_name &name = *iter;

		sccs_file file(name, sccs_file::UPDATE);
		if (file.failed()) {
			return_status = 1;
			continue;
		}
		
		sccs_pfile pfile(name, sccs_pfile::UPDATE);
		if (pfile.failed()) {
			return_status = 1;
			continue;
		}

		if (first) {
			if (stdin_is_a_tty()) {
				if (mrs == NULL && file.mr_required()) {
					mrs = prompt_user(msgptr(21, "MRs? "));
				}
				if (comments == NULL) {
					comments = prompt_user(
						msgptr(26, "comments? "));
				}
			}
			mr_list = split_mrs(mrs);
			comment_list = split_comments(comments);
			first = 0;
		}

		switch(pfile.find_sid(rid)) {
		case sccs_pfile::FOUND:
			break;

		case sccs_pfile::NOT_FOUND:
			if (null_delta) {
				break;
			}

			if (rid == NULL) {
				error(msgptr(28, "%s: You have no edits"
					     " outstanding."),
				      (char const *) name);
			} else {
				error(msgptr(29, "%s: Requested SID hasn't"
					     " been locked for editing"
					     " by you."),
					     (const char *) name);
			}
			return_status = 1;
			continue;

		case sccs_pfile::AMBIGUOUS:
			if (rid != NULL) {
				error(msgptr(30, "%s: Requested SID"
					     " is ambiguous."),
					     (char const *) name);
			} else {
				error(msgptr(31, "%s: You must specify a SID"
					     " on the command line."),
					     (char const *) name);
			}
			return_status = 1;
			continue;

		default:
			abort();
		}

		if (file.mr_required()) {
			if (mr_list.length() == 0) {
				error(msgptr(22, "%s: MR number(s) must"
					     " be supplied."),
				      (char const *) name);
				return_status = 1;
				continue;
			}
			if (file.check_mrs(mr_list)) {
				error(msgptr(23, "%s: Invalid MR number(s)."),
				      (char const *) name);
				return_status = 1;
				continue;
			}
		}

		string gname = name.gfile();

		int r;
		if (null_delta) {
			r = file.add_delta(NULL, pfile, mr_list, comment_list);
		} else {
			r = file.add_delta(gname, pfile,
					   mr_list, comment_list);
		}
		if (r == -1) {
			return_status = 1;
			continue;
		}

		if (!keep_gfile) {
#ifndef TESTING
			remove(gname);
#endif
		}
	}

	return return_status;
}

/* Local variables: */
/* mode: c++ */
/* End: */
