/*
 * msgdb.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Functions for handling message string databases.
 *
 */

#include "mysc.h"

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) MySC msgdb.c 1.7 00/05/12 14:44:22";
#endif

#ifdef CONFIG_ENABLE_MSGDB

static inline char const *
cache_message(int num, char const *msg) {
	const int bufferlen = 5;
	static char *buffer[bufferlen];
	static int bufferpos = 0;
	
	if (++bufferpos == bufferlen) {
		bufferpos = 0;
	}

	char *p = buffer[bufferpos];
	if (p == NULL) {
		free(p);
	}
		
	char tmp[20];
	sprintf(tmp, "[mysc%03d] ", num);
	return buffer[bufferpos] = strcat(strcpy((char *) xmalloc(strlen(tmp) 
						                  + strlen(msg)
								  + 1),
				                 tmp), msg);
}

#if !defined(CONFIG_NO_CATGETS)

/*
 * X/Open catgets() bases implementation.
 */

#include <nl_types.h>

static nl_catd msgcatid = (nl_catd) -1;

void
init_msgdb() {
#ifdef NL_CAT_LOCALE
	msgcatid = catopen("MySC", NL_CAT_LOCALE);
#else
	msgcatid = catopen("MySC", 0);
#endif
}

char const *
do_msgptr(int num, char const *msg) {
	if (num == 0 || msgcatid == (nl_catd) -1) {
		return msg;
	}

	int save_errno = errno;
	char const *str = catgets(msgcatid, NL_SETD, num, (char *) msg);
	if (str != msg) {
		msg = cache_message(num, str);
	}
	errno = save_errno;
	return msg;
}

#elif !defined(CONFIG_NO_GETTEXT)

/*
 *  Sun gettext() based implementation.
 */ 

#include <libintl.h>

void
init_msgdb() {
	return;
}

char const *
do_msgptr(int num, char const *msg) {
	if (num == 0) {
		return msg;
	}
	int save_errno = errno;
	char const *str = dgettext("MySC", msg);
	if (str != msg) {
	 	msg = cache_message(num, str);
	}
	errno = save_errno;
	return msg;
}

#elif !defined(CONFIG_NO_GETTXT)

/*
 * System V R4 gettxt() implementation.
 */

#include <nl_types.h>

void
init_msgdb() {
	return;
}

char const *
do_msgptr(int num, char const *nstr, char const *msg) {
	if (num == 0) {
		return msg;
	}
	int save_errno = errno;
	char const *str = gettxt(nstr, msg);
	if (str != msg) {
		msg = cache_message(num, str);
	}
	return msg;
}

#else

/*
 * No message database implementation.
 */

void
init_msgdb() {
	return;
}

char const *
do_msgptr(int num, char const *msg) {
	int save_errno = errno;
	msg = cache_message(num, msg);
	errno = save_errno;
	return msg;
}

#endif

#else /* CONFIG_ENABLE_MSGDB */

/*
 * Non-implementation.
 */

void
init_msgdb() {
	return;
}

#endif /* CONFIG_ENABLE_MSGDB */

