/*
 * prs.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Prints selected parts of an SCCS file.
 *
 */

#include "mysc.h"
#include "version.h"
#include "fileiter.h"
#include "sccsfile.h"
#include "getopt.h"

char const main_sccs_id[] = "@(#) MySC prs.c 1.10 98/04/21 16:44:19";

void
usage() {
	fprintf(stderr, msgptr(72,
"usage: %s [-aelDRV] [-c cutoff] [-d format] [-r SID] file ...\n"),
		prg_name);
}

int
main(int argc, char **argv) {
	int c;
	string format = msgptr(73, ":Dt:\t:DL:\nMRs:\n:MR:COMMENTS:\n:C:");
	sid rid = NULL;
	/* enum */ sccs_file::when when = sccs_file::SIDONLY;
	int all_deltas = 0;
	sccs_date cutoff;
	int default_processing = 1;
	int return_status = 0;

	if (argc > 0) {
		set_prg_name(argv[0]);
	} else {
		set_prg_name("prs");
	}

	setlocale(LC_ALL, "");
	init_msgdb();

	class getopt opts(argc, argv, "d?Dr?Relc:aV");
	for(c = opts.next(); c != getopt::END_OF_ARGUMENTS; c = opts.next()) {
		switch (c) {
		default:
			quit(-2, msgptr(10, "Unsupported option: '%c'"), c);

		case 'd':
			if (opts.getarg() != NULL) {
				format = opts.getarg();
			}
			/* FALLTHROUGH */
		case 'D':
			default_processing = 0;
			break;

		case 'r':
			rid = sid(opts.getarg());
			if (rid != NULL && !rid.valid()) {
				quit(-2, msgptr(11, "Invalid SID: '%s'"),
				     opts.getarg());
			}
			default_processing = 0; /* */
			break;

		case 'R':
			rid = NULL;
			default_processing = 0; /* */
			break;
			
		case 'c':
			cutoff = sccs_date(opts.getarg());
			if (!cutoff.valid()) {
				quit(-2,
				     msgptr(12, "Invalid cutoff date: '%s'"),
				     opts.getarg());
			}
			break;

		case 'e':
			when = sccs_file::EARLIER;
			default_processing = 0; /* */
			break;

		case 'l':
			when = sccs_file::LATER;
			default_processing = 0; /* */
			break;

		case 'a':
			all_deltas = 1;
			break;

		case 'V':
			version();
			break;
		}

	}

	if (when == sccs_file::SIDONLY && cutoff.valid()) {
		quit(-2, msgptr(74, "Either the -e or -l switch must used with"
				" a cutoff date."));
	}

	if (default_processing) {
		when = sccs_file::EARLIER;
	}

	sccs_file_iterator iter(argc, argv, opts.get_index());
	if (iter.length_zero()) {
		quit(-2, msgptr(16, "No SCCS file specified."));
	}

	for(; iter != sccs_file_iterator(); ++iter) {
		sccs_name &name = *iter;
		sccs_file file(name, sccs_file::READ);
		if (file.failed()) {
			return_status = 1;
			continue;
		}

		if (default_processing) {
			if (printf("%s:\n\n", (char const *) name) == EOF) {
				error(errno, msgptr(42, "%s: Write error."),
				      "-");
				return_status = 1;
				continue;
			}
		}
#ifdef GNUC_BUG_2_7_2_3
		sccs_file::when tmp = when;
		if (file.prs(stdout, format, rid, cutoff, tmp, all_deltas) 
		    == -1) {
			return_status = 1;
		}
#else
		if (file.prs(stdout, format, rid, cutoff, when, all_deltas)
		    == -1) {
			return_status = 1;
		}
#endif
	}

	return return_status;
}

/* Local variables: */
/* mode: c++ */
/* End: */
