/*
 * quit.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Functions for cleaning up and quitting.
 *
 */

#include "mysc.h"
#include "sysdep.h"

#include <stdarg.h>

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) MySC quit.c 1.10 98/01/27 04:50:18";
#endif

#ifdef CONFIG_BORLANDC
#ifdef __STDC__
extern "C" char * CDECL strlwr(char *);
#endif
#include <dir.h>
#endif /* CONFIG_BORLANDC */

char const *prg_name = NULL;

#ifdef CONFIG_DECLARE_STRERROR
extern "C" char * CDECL strerror(int err);
#endif

void
set_prg_name(char const *name) {
#ifdef CONFIG_BORLANDC

	static char file[MAXFILE + MAXEXT];
	char ext[MAXEXT];

	fnsplit(name, NULL, NULL, file, ext);
	strlwr(file);
	strlwr(ext);

	if (strcmp(ext, ".exe") != 0 && strcmp(ext, ".com") != 0) {
		strcat(file, ext);
	}
	prg_name = file;

#else /* CONFIG_BORLANDC */

	prg_name = name;

#endif /* CONFIG_BORLANDC */
}


static void 
do_error(int err, char const *fmt, va_list ap) {

	if (err == -2) {
		usage();
	}

	if (prg_name != NULL && err != 0) {
		fprintf(stderr, "%s: ", (char const *) prg_name);
	}

	vfprintf(stderr, fmt, ap);

	putc('\n', stderr);

	if (err >= 1) {
#ifdef CONFIG_NO_STRERROR
		if (err <= sys_nerr) {
			fprintf(stderr, "%d - %s\n", err, sys_errlist[err]);
		} else {
			fprintf(stderr, msgptr(75, "%d - Unknown error\n")
				, err);
		}
#else
		fprintf(stderr, "%d - %s\n", err, strerror(err));
#endif
	}

	fflush(stderr);
}

static void do_quit(int err, int ret, char const *fmt, va_list ap) NORETURN;

static void 
do_quit(int err, int ret, char const *fmt, va_list ap) {
	fflush(stdout);
	cleanup::run_cleanups();

	do_error(err, fmt, ap);

	fflush(stdout);
#ifndef CONFIG_NO_ABORT
	if (err == -3) {
		abort();
	}
#endif

#ifndef CONFIG_NO_PIPE
	if (cleanup::in_child()) {
		_exit(ret);
	}
#endif
	exit(ret);
}

void 
quit(int err, char const *fmt, ...) {
	int ret;

	va_list ap;

	va_start(ap, fmt);

	if (err > 0) {
		ret = 1;
	} else {
		ret = -err;
	}

	do_quit(err, ret, fmt, ap);

	va_end(ap);
}

void 
quit(int err, int ret, char const *fmt, ...) {
	va_list ap;

	va_start(ap, fmt);

	do_quit(err, ret, fmt, ap);

	va_end(ap);
}

void 
error(char const *fmt, ...) {
	va_list ap;

	va_start(ap, fmt);

	do_error(-1, fmt, ap);

	va_end(ap);
}

void 
error(int err, char const *fmt, ...) {
	va_list ap;

	va_start(ap, fmt);

	do_error(err, fmt, ap);

	va_end(ap);
}

void
nomem() {
	static int count = 0;

	switch(count++) {
	case 0:
		quit(-4, msgptr(76, "Out of memory!"));

	case 1:
		quit(-4, "Out of memory!");

	default:
		abort();
	}
}

void
assert_failed(char const *file, int line, char const *test) {
	quit(-3, msgptr(77, "Assertion failed: %s\nFile: %s  Line: %d"),
	     test, file, line);
}


class cleanup *cleanup::head = NULL;
int cleanup::running = 0;
int cleanup::all_disabled = 0;
#ifndef CONFIG_NO_FORK
int cleanup::in_child_flag = 0;
#endif

cleanup::cleanup() {
	next = head;
	head = this;
}

cleanup::~cleanup() {
	class cleanup *p = head;

	assert(head != NULL);

	if (p == this) {
		head = next;
		return;
	}
	
	while(p->next != this) {
		p = p->next;
		assert(p != NULL);
	}

	p->next = next;
}		

void
cleanup::run_cleanups() {
	if (running || all_disabled) {
		return;
	}
	running = 1;

	class cleanup *p = head;
	while(p != NULL) {
		if (p->do_cleanup()) {
			head = p->next;
			running = 0;
			return;
		}
		p = p->next;
	}

	running = 0;
	all_disabled++;
	return;
}

/* Local variables: */
/* mode: c++ */
/* End: */
