/*
 * sf-get4.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Members of the class sccs_file only used by get and sccsdiff.
 *
 */

#include "mysc.h"
#include "sccsfile.h"
#include "seqstate.h"

#ifdef CONFIG_SCCS_IDS
static char const sccs_id[] = "@(#) MySC sf-get4.c 1.1 98/06/06 15:41:05";
#endif

/* Returns the SID of the delta to retrieve that best matches the
   requested SID. */

sid
sccs_file::find_requested_sid(sid requested) const {
	if (requested == NULL) {
		requested = flags.default_sid;
	}

	sid best = NULL;

	/* Find the delta with the highest SID that matches the
	   requested SID */

	delta_iterator iter(delta_table);

	if (requested.partial_sid()) {
		while(iter.next()) {
			sid const &id = iter->id;
			if (requested.partial_match(id)
			    && (id > best || best == NULL)) {
				best = id;
			}
		}
	} else {
		while(iter.next()) {
			if (iter->id == requested) {
				return requested;
			}
		}
	}

	if (best != NULL || !requested.release_only()) {
		return best;
	}

	/* If a match wasn't found above and the requested SID
	   only mentions a release then look for the delta with 
           the highest SID that's lower than the requested SID. */

	best = sid(NULL);

	iter.rewind();
	while(iter.next()) {
		sid const &id = iter->id;
		if (id > best && id < requested) {
			best = id;
		}
	}

	return best;
}


/* Write a line of a file after substituting any id keywords in it.
   Returns true if an error occurs. */

int
sccs_file::print_subst(char const *start, struct subst_parms *parms) const {
	struct delta const &delta = parms->delta;
	FILE *out = parms->out;

	char const *percent = strchr(start, '%');
	while(percent != NULL) {
		char c = percent[1];
		if (c != '\0' && percent[2] == '%') {
			if (start != percent
			    && fwrite(start, percent - start, 1, out) != 1) {
				return EOF;
			}

			percent += 3;

			int err = 0;

			switch(c) {
				char const *s;

			case 'M':
			{
				string module = get_module_name();
				err = fputs(module, out);
			}
				break;
			
			case 'I':
				err = delta.id.print(out);
				break;

			case 'R':
				err = delta.id.printf(out, 'R');
				break;

			case 'L':
				err = delta.id.printf(out, 'L');
				break;

			case 'B':
				err = delta.id.printf(out, 'B');
				break;

			case 'S':
				err = delta.id.printf(out, 'S');
				break;

			case 'D':
				err = parms->now.printf(out, 'D');
				break;
				
			case 'H':
				err = parms->now.printf(out, 'H');
				break;

			case 'T':
				err = parms->now.printf(out, 'T');
				break;

			case 'E':
				err = delta.date.printf(out, 'D');
				break;

			case 'G':
				err = delta.date.printf(out, 'H');
				break;

			case 'U':
				err = delta.date.printf(out, 'T');
				break;

			case 'Y':
				s = flags.type;
				if (s != NULL) {
					err = fputs(s, out);
				}
				break;

			case 'F':
				err = fputs(base_part(name), out);
				break;

			case 'P':
				err = fputs(name, out);
				break;

			case 'Q':
				s = flags.user_def;
				if (s != NULL) {
					err = fputs(s, out);
				}
				break;

			case 'C':
				err = fprintf(out, "%d",
					      parms->out_lineno);
				break;

			case 'Z':
				if (fputc('@', out) == EOF
				    || fputs("(#)", out) == EOF) {
					err = EOF;
				} else {
					err = 0;
				}
				break;

			case 'W':
				s = parms->wstring;
				if (s == NULL) {
					s = "%Z""%%M""%\t%""I%";
				} else {
					/* protect against recursion */
					parms->wstring = NULL; 
				}
				err = print_subst(s, parms);
				if (parms->wstring == NULL) {
					parms->wstring = s;
				}
				break;

			case 'A':
				err = print_subst("%Z""%%Y""% %M""% %I"
						  "%%Z""%", parms);
				break;

			default:
				start = percent - 3;
				percent = percent - 1;
				continue;
			}

			parms->found_id = 1;

			if (err == EOF) {
				return EOF;
			}
			start = percent;
		} else {
			percent++;
		}
		percent = strchr(percent, '%');
	}

	return fputs(start, out);
}

/* Output the specified version to a file with possible modifications.
   Most of the actual work is done with a seqstate object that 
   figures out whether or not given line of the SCCS file body
   should be included in the output file. */
	
/* struct */ sccs_file::get_status
sccs_file::get(FILE *out, string gname, sid id, sccs_date cutoff,
	       sid_list include, sid_list exclude,
	       int keywords, char const *wstring,
	       int show_sid, int show_module) {
	struct get_status status;

	seq_state state(delta_table.highest_seqno());
	struct delta const *delta = delta_table.find(id);
	assert(delta != NULL);

	
	if (prepare_seqstate(state, delta->seq) == -1) {
		status.failed = -1;
		return status;
	}

	prepare_seqstate(state, include, exclude, cutoff);

	struct subst_parms parms(out, wstring, *delta, 0, sccs_date::now());

	if (keywords) {
		status.failed = get(gname, state, parms, NULL,
				    show_sid, show_module);
	} else {
		status.failed = get(gname, state, parms,
				    &sccs_file::print_subst, 
				    show_sid, show_module);
	}

	if (status.failed == -1) {
		return status;
	}

	if (!parms.found_id) {
		if (flags.id_keywords != NULL
		    && *(char const *)flags.id_keywords != '\0') {
			fprintf(stderr,
				msgptr(144, "%s: Warning: Required keywords"
				       " \"%s\" missing."),
				(char const *) name,
				(char const *) flags.id_keywords);
		}
		fprintf(stderr, msgptr(119, "%s: Warning: No id keywords.\n"),
			(char const *) name);
	}
				     
	/* Set the return status. */
	status.lines = parms.out_lineno;
	
	seq_no seq;	
	for(seq = 1; seq <= delta_table.highest_seqno(); seq++) {
		if (state.is_explicit(seq)) {
			if (state.is_included(seq)) {
				status.included.add(delta_table[seq].id);
			} else if (state.is_excluded(seq)) {
				status.excluded.add(delta_table[seq].id);
			}
		}
	}
			
	return status;
}

/* Local variables: */
/* mode: c++ */
/* End: */
