/*
 * unget.c
 *
 * By Ross Ridge
 * Public Domain
 *
 * Removes edit locks from SCCS files.
 *
 */

#include "mysc.h"
#include "version.h"
#include "fileiter.h"
#include "pfile.h"
#include "getopt.h"

char const main_sccs_id[] = "@(#) MySC unget.c 1.7 98/03/18 22:23:25";

void
usage() {
	fprintf(stderr, msgptr(171,
"usage: %s [-nsV] [-r SID] file ...\n"),
		prg_name);
}

int
main(int argc, char **argv) {
	int c;
	sid rid = NULL;
	int silent = 0;
	int keep_gfile = 0;
	int return_status = 0;

	if (argc > 0) {
		set_prg_name(argv[0]);
	} else {
		set_prg_name("unget");
	}

	setlocale(LC_ALL, "");
	init_msgdb();

	class getopt opts(argc, argv, "r:snV");
	for(c = opts.next(); c != getopt::END_OF_ARGUMENTS; c = opts.next()) {
		switch (c) {
		default:
			quit(-2, msgptr(10, "Unsupported option: '%c'"), c);

		case 'r':
			rid = sid(opts.getarg());
			if (!rid.valid()) {
				quit(-2, msgptr(11, "Invalid SID: '%s'"),
				     opts.getarg());
			}
			break;

		case 's':
			silent = 1;
			break;

		case 'n':
			keep_gfile = 1;
			break;

		case 'V':
			version();
			break;
		}
	}

     	if (silent) {
		stdout_to_null();
	}

	sccs_file_iterator iter(argc, argv, opts.get_index());
	if (iter.length_zero()) {
		quit(-2, msgptr(16, "No SCCS file specified."));
	}

	for(; iter != sccs_file_iterator(); ++iter) {
		sccs_name &name = *iter;
		sccs_pfile pfile(name, sccs_pfile::UPDATE);
		if (pfile.failed()) {
			return_status = 1;
			continue;
		}

		switch(pfile.find_sid(rid)) {
		case sccs_pfile::FOUND:
			break;

		case sccs_pfile::NOT_FOUND:
			if (rid == NULL) {
				error(msgptr(28, "%s: You have no edits"
					     " outstanding."),
				      (char const *) name);
			} else {
				error(msgptr(29, "%s: Requested SID hasn't"
					     " been locked for editing"
					     " by you."),
				      (const char *) name);
			}
			return_status = 1;
			continue;

		case sccs_pfile::AMBIGUOUS:
			if (rid != NULL) {
				error(msgptr(30, "%s: Requested SID"
					     " is ambiguous."),
				      (char const *) name);
			} else {
				error(msgptr(31, "%s: You must specify a"
					     " SID on the command line."),
					     (char const *) name);
			}
			return_status = 1;
			continue;

		default:
			abort();
		}

		pfile.delete_lock();
		if (pfile.update() == -1) {
			return_status = 1;
			continue;
		}

		if (!keep_gfile) {
			string gname = name.gfile();
#ifndef TESTING
			if (remove(gname) == -1) {
				error(msgptr(172, "%s: Couldn't remove file."),
				      (char const *) gname);
				return_status = 1;
				continue;
			}
#endif
		}
	}

	return return_status;
}

/* Local variables: */
/* mode: c++ */
/* End: */
